@@ -212,8 +212,8 @@ def read_dx(filename):
 
     Parameters
     ----------
-    filename : string
-        binary file of DX raw data
+    filename : string or file-like
+        filename of binary file of DX raw data or file-like object
 
     Returns
     -------
@@ -249,48 +249,44 @@ def read_dx(filename):
     clutterflag = 2 ** 15
     dataflag = 2 ** 13 - 1
 
-    f = get_radolan_filehandle(filename)
+    with get_radolan_filehandle(filename) as f:
 
-    # header string for later processing
-    header = ""
-    atend = False
+        # header string for later processing
+        header = ""
+        atend = False
 
-    # read header
-    while True:
-        mychar = f.read(1)
-        # 0x03 signals the end of the header but sometimes there might be
-        # an additional 0x03 char after that
+        # read header
+        while True:
+            mychar = f.read(1)
+            # 0x03 signals the end of the header but sometimes there might be
+            # an additional 0x03 char after that
 
-        if mychar == b"\x03":
-            atend = True
-        if mychar != b"\x03" and atend:
-            break
-        header += str(mychar.decode())
+            if mychar == b"\x03":
+                atend = True
+            if mychar != b"\x03" and atend:
+                break
+            header += str(mychar.decode())
 
-    attrs = parse_dx_header(header)
+        attrs = parse_dx_header(header)
 
-    # position file at end of header
-    f.seek(len(header))
+        # position file at end of header
+        f.seek(len(header))
 
-    # read number of bytes as declared in the header
-    # intermediate fix:
-    # if product length is uneven but header is even (e.g. because it has two
-    # chr(3) at the end, read one byte less
-    buflen = attrs["bytes"] - len(header)
-    if (buflen % 2) != 0:
-        # make sure that this is consistent with our assumption
-        # i.e. contact DWD again, if DX files show up with uneven byte lengths
-        # *and* only one 0x03 character
-        # assert header[-2] == chr(3)
-        buflen -= 1
+        # read number of bytes as declared in the header
+        # intermediate fix:
+        # if product length is uneven but header is even (e.g. because it has two
+        # chr(3) at the end, read one byte less
+        buflen = attrs["bytes"] - len(header)
+        if (buflen % 2) != 0:
+            # make sure that this is consistent with our assumption
+            # i.e. contact DWD again, if DX files show up with uneven byte lengths
+            # *and* only one 0x03 character
+            # assert header[-2] == chr(3)
+            buflen -= 1
 
-    buf = f.read(buflen)
-    # we can interpret the rest directly as a 1-D array of 16 bit unsigned ints
-    raw = np.frombuffer(buf, dtype="uint16")
-
-    # reading finished, close file, but only if we opened it.
-    if isinstance(f, io.IOBase):
-        f.close()
+        buf = f.read(buflen)
+        # we can interpret the rest directly as a 1-D array of 16 bit unsigned ints
+        raw = np.frombuffer(buf, dtype="uint16")
 
     # a new ray/beam starts with bit 14 set
     # careful! where always returns its results in a tuple, so in order to get
@@ -615,7 +611,7 @@ def decode_radolan_runlength_array(binarr, attrs):
     return np.flipud(arr)
 
 
-def read_radolan_binary_array(fid, size):
+def read_radolan_binary_array(fid, size, raise_on_error=True):
     """Read binary data from file given by filehandle
 
     Parameters
@@ -624,6 +620,8 @@ def read_radolan_binary_array(fid, size):
         file handle
     size : int
         number of bytes to read
+    raise_on_error : bool
+        raise IOError if data is truncated
 
     Returns
     -------
@@ -631,11 +629,14 @@ def read_radolan_binary_array(fid, size):
         array of binary data
     """
     binarr = fid.read(size)
-    fid.close()
-    if len(binarr) != size:
+    if raise_on_error and len(binarr) != size:
+        try:
+            desc = fid.name
+        except AttributeError:
+            desc = repr(fid)
         raise IOError(
             "{0}: File corruption while reading {1}! \nCould not "
-            "read enough data!".format(__name__, fid.name)
+            "read enough data!".format(__name__, desc)
         )
     return binarr
 
@@ -645,29 +646,31 @@ def get_radolan_filehandle(fname):
 
     Parameters
     ----------
-    fname : string
-        filename
+    fname : string or file-like
+        filename or file-like object
 
     Returns
     -------
     f : object
-        filehandle
+        file handle
     """
-
-    gzip = util.import_optional("gzip")
-
-    # open file handle
-    try:
-        f = gzip.open(fname, "rb")
-        f.read(1)
-    except IOError:
-        f = open(fname, "rb")
-        f.read(1)
-
-    # rewind file
-    f.seek(0, 0)
-
-    return f
+    ret = lambda obj: obj
+    if isinstance(fname, str):
+        gzip = util.import_optional("gzip")
+        # open file handle
+        try:
+            with gzip.open(fname, "rb") as f:
+                f.read(1)
+                f.seek(0, 0)
+                ret = gzip.open
+        except IOError:
+            with open(fname, "rb") as f:
+                f.read(1)
+                f.seek(0, 0)
+                ret = open
+        return ret(fname, "rb")
+    else:
+        return ret(fname)
 
 
 def read_radolan_header(fid):
@@ -694,7 +697,7 @@ def read_radolan_header(fid):
     return header
 
 
-def read_radolan_composite(f, missing=-9999, loaddata=True):
+def read_radolan_composite(f, missing=-9999, loaddata=True, fillmissing=False):
     """Read quantitative radar composite format of the German Weather Service
 
     The quantitative composite format of the DWD (German Weather Service) was
@@ -723,13 +726,16 @@ def read_radolan_composite(f, missing=-9999, loaddata=True):
 
     Parameters
     ----------
-    f : string or file handle
-        path to the composite file or file handle
+    f : string or file-like
+        path to the composite file or file-like object
     missing : int
         value assigned to no-data cells
     loaddata : bool | str
         True | False | 'xarray', If False function returns (None, attrs)
         If 'xarray' returns (xarray Dataset, attrs)
+    fillmissing : bool
+        If True fills truncated values with "missing". Defaults to False.
+        Does not work for run-length encoded files ("PC" and "PG).
 
     Returns
     -------
@@ -746,38 +752,51 @@ def read_radolan_composite(f, missing=-9999, loaddata=True):
 
     NODATA = missing
     mask = 0xFFF  # max value integer
-    close = False
 
-    # If a file name is supplied, get a file handle
-    if isinstance(f, str):
-        f = get_radolan_filehandle(f)
-        close = True
+    # get file handle
+    with get_radolan_filehandle(f) as fid:
 
-    header = read_radolan_header(f)
-    attrs = parse_dwd_composite_header(header)
+        header = read_radolan_header(fid)
+        attrs = parse_dwd_composite_header(header)
 
-    if not loaddata:
-        if close:
-            f.close()
-        return None, attrs
+        if not loaddata:
+            # if close:
+            #     f.close()
+            return None, attrs
 
-    attrs["nodataflag"] = NODATA
+        attrs["nodataflag"] = NODATA
 
-    if not attrs["radarid"] == "10000":
-        warnings.warn(
-            "WARNING: You are using function e"
-            + "wradlib.io.read_RADOLAN_composit for a non "
-            + "composite file.\n "
-            + "This might work...but please check the validity "
-            + "of the results"
-        )
+        if not attrs["radarid"] == "10000":
+            warnings.warn(
+                "WARNING: You are using function e"
+                + "wradlib.io.read_RADOLAN_composit for a non "
+                + "composite file.\n "
+                + "This might work...but please check the validity "
+                + "of the results"
+            )
 
-    # read the actual data
-    indat = read_radolan_binary_array(f, attrs["datasize"])
+        # handle truncated data
+        binarr_kwargs = {}
+        if fillmissing and attrs["producttype"] not in ["PG", "PC"]:
+            binarr_kwargs.update(dict(raise_on_error=False))
+
+        # read the actual data
+        indat = read_radolan_binary_array(fid, attrs["datasize"], **binarr_kwargs)
+
+        # helper function to fill truncated data with 'nodata' values
+        def _from_buffer(data, size, dtype):
+            if len(data) < size:
+                isize = np.dtype(dtype).itemsize
+                fill_value = 250 if isize == 1 else 8192
+                if len(data) % isize:
+                    data = data[:-1]
+                fill = np.full((size - len(data)) // isize, fill_value, dtype=dtype)
+                data += fill.tobytes()
+            return np.frombuffer(data, dtype).astype(dtype)
 
     if attrs["producttype"] in ["RX", "EX", "WX"]:
         # convert to 8bit integer
-        arr = np.frombuffer(indat, np.uint8).astype(np.uint8)
+        arr = _from_buffer(indat, attrs["datasize"], np.uint8)
         attrs["nodatamask"] = np.where(arr == 250)[0]
         attrs["cluttermask"] = np.where(arr == 249)[0]
     elif attrs["producttype"] in ["PG", "PC"]:
@@ -785,7 +804,7 @@ def read_radolan_composite(f, missing=-9999, loaddata=True):
         attrs["nodatamask"] = np.where(arr == 255)[0]
     else:
         # convert to 16-bit integers
-        arr = np.frombuffer(indat, np.uint16).astype(np.uint16)
+        arr = _from_buffer(indat, attrs["datasize"], np.uint16)
         # evaluate bits 13, 14, 15 and 16
         attrs["secondary"] = np.where(arr & 0x1000)[0]
         attrs["nodatamask"] = np.where(arr & 0x2000)[0]
@@ -812,9 +831,6 @@ def read_radolan_composite(f, missing=-9999, loaddata=True):
         if "nodatamask" in attrs:
             arr.flat[attrs["nodatamask"]] = NODATA
 
-    if close:
-        f.close()
-
     return arr, attrs
 
 
